
/*************************************************************************************************************************/
/*                                                                                                                       */
/*                                                                                                                       */
/*                                                    S H E D U L E R                                                    */
/*                                                                                                                       */
/*                                                                                                                       */
/*                                                                                                                       */
/*  !!!(Sheduler for MCC30 is in "port\osa_mcc30.h")!!!                                                                  */
/*                                                                                                                       */
/*                                                                                                                       */
/*                                                                                                                       */
/*************************************************************************************************************************/



#ifndef OS_Sched                                                        

//------------------------------------------------------------------------------
//    (  )
//          ,  
// ,         , 
//   .



//------------------------------------------------------------------------------
#ifdef OS_ENABLE_CRITICAL_SECTION
    #define SKIP_IF_IS_IN_CRITICAL_SECTION()    if (OS_IsInCriticalSection()) goto SCHED_CHECK_READY;
    #define LABEL_SCHED_CHECK_READY             SCHED_CHECK_READY:
    #define NOT_IN_CRITICAL_SECTION()           !OS_IsInCriticalSection()
#else
    #define SKIP_IF_IS_IN_CRITICAL_SECTION()
    #define LABEL_SCHED_CHECK_READY 
    #define NOT_IN_CRITICAL_SECTION()           1
#endif


//------------------------------------------------------------------------------
//   MCC30
#if defined(__OSAMCC30__)

    #define LABEL_SCHED_CONTINUE            asm("   SCHED_CONTINUE:         ");
    #define IF_NOT_TASK_ENABLE_CONTINUE()   asm("   btss    _OS_state, #6   "); \
                                            asm("   bra     SCHED_CONTINUE  ");

    #define LABEL_SCHED_SKIP_RUN            asm("   SCHED_SKIP_RUN:         ");
    #define IF_NOT_TASK_READY()             asm("   btsc    _OS_state, #3   "); \
                                            asm("   bra     SCHED_SKIP_RUN  ");

    #define IF_TASK_READY()                 asm("   btss    _OS_state, #3   "); \
                                            asm("   bra     SCHED_CONTINUE  ");


#else
    #define IF_NOT_TASK_ENABLE_CONTINUE()   if (!_OS_bTaskEnable) continue;
    #define LABEL_SCHED_CONTINUE    
    #define IF_NOT_TASK_READY()             if (!_OS_bTaskReady)
    #define IF_TASK_READY()                 if (_OS_bTaskReady)
    #define LABEL_SCHED_SKIP_RUN
#endif


#ifdef OS_ENABLE_TTIMERS
    #define IF_NOT_OS_CHECK_DELAYS_GOTO_SCHED_CONTINUE  if (_OS_bTaskDelay && !_OS_bTaskCanContinue) continue;
#else
    #define IF_NOT_OS_CHECK_DELAYS_GOTO_SCHED_CONTINUE
#endif



//------------------------------------------------------------------------------
#if defined(__OSAMCC30__) || defined(__OSAPICC12__) || defined(__OSAPICC18__)
    #define _OS_SET_STATE()     OS_state = _OS_CurTask->State
    #define _OS_RESTORE_INDF()
#else
    #define _OS_SET_STATE()     _OS_SET_FSR_CUR_TASK()
    #define _OS_RESTORE_INDF()  _OS_SET_FSR_CUR_TASK()
#endif

#if !defined(OS_DISABLE_PRIORITY)


    //      ,  
    //      
    #define OS_IsIdle()     (!OS_IsInCriticalSection() && !OS_Flags.bBestTaskFound)

/************************************************************************/
/*                                                                      */
/************************************************************************/
    #define OS_Sched()                                                                                  \
    {                                                                                               \
        OS_Flags.bBestTaskFound = 0;               /*  ,     */ \
        OS_Flags.bCheckingTasks = 1;               /*          */ \
        _OS_best_priority = OS_WORST_PRIORITY + 1; /*               */ \
        SKIP_IF_IS_IN_CRITICAL_SECTION();                                                           \
        _OS_CurTask = _OS_LastTask;              /*   ,    */ \
        do                                                                                          \
        {                                                                                           \
            if (_OS_CurTask >= (OST_TASK_POINTER) &OS_TaskVars[OS_TASKS-1])                         \
                _OS_CurTask = (OST_TASK_POINTER) &OS_TaskVars[0];                                   \
            else                                                                                    \
                _OS_CurTask++;                                                                      \
                                                                                                    \
    LABEL_SCHED_CHECK_READY                                                                         \
            _OS_SET_STATE(); /*  16  MCC18   INDF,  30, 12, PICC18 -  OS_state */ \
            IF_NOT_TASK_ENABLE_CONTINUE();    /* ,            */       \
            if (_OS_best_priority <= (_OS_temp = _OS_cTaskPriority)) continue; /* . .1  */  \
            IF_NOT_TASK_READY()                                                                     \
            {                                                                                       \
                /*      .   */                       \
    SCHED_RUN:;                                                                                     \
                _OS_JumpToTask();                                                                   \
                _OS_SaveRetAddr();                                                                  \
                if (!OS_Flags.bCheckingTasks) goto SCHED_END;                                       \
                _OS_RESTORE_INDF();                                                                 \
            }                                                                                       \
    LABEL_SCHED_SKIP_RUN                                                                            \
            IF_NOT_OS_CHECK_DELAYS_GOTO_SCHED_CONTINUE;  /* ,        */ \
            IF_TASK_READY()                         /*        */ \
            {                                                                                       \
                OS_Flags.bBestTaskFound = 1;                                                        \
                _OS_BestTask = _OS_CurTask;                                                         \
                _OS_best_priority = _OS_cTaskPriority;                                              \
                /*   ,      */                            \
                if (_OS_best_priority == OS_BEST_PRIORITY) break;                                   \
            }                                                                                       \
    LABEL_SCHED_CONTINUE                                                                            \
        } while (NOT_IN_CRITICAL_SECTION() && _OS_CurTask != _OS_LastTask);                         \
        /*      ,    */                                \
        if (OS_Flags.bBestTaskFound)                                                                \
        {                                                                                           \
            OS_Flags.bCheckingTasks = 0;     /*       */      \
            _OS_LastTask = _OS_CurTask = _OS_BestTask;                                              \
            goto SCHED_RUN;                                                                         \
        }                                                                                           \
    SCHED_END:;                                                                                     \
    }                                                                                               \
/************************************************************************
     OS_Sched()

1.  OS_Temp  ,    RAM  1 .   
    _OS_best_priority  Indf & 7   .    (_OS_temp) 
         .


 ************************************************************************/
#endif


#if defined(OS_DISABLE_PRIORITY)

    //      ,  
    //      
    // (     0)
    #define OS_IsIdle()     0

    //------------------------------------------------------------------------------
    //    (  )
    //     ,     
    //    . ,    ,
    //   .       
    //   OS_TASKS     .


    #ifdef OS_ENABLE_CRITICAL_SECTION
        #define IF_NOT_IS_IN_CRITICAL_SECTION()        if (!OS_IsInCriticalSection())
    #else
        #define IF_NOT_IS_IN_CRITICAL_SECTION()
    #endif

    #define OS_Sched()                                                              \
        {                                                                           \
            _OS_temp = 0;                                                           \
            IF_NOT_IS_IN_CRITICAL_SECTION()                                         \
            {                                                                       \
                if (_OS_CurTask >= (OST_TASK_POINTER) &OS_TaskVars[OS_TASKS-1])     \
                    _OS_CurTask = (OST_TASK_POINTER) &OS_TaskVars[0];               \
                else                                                                \
                    _OS_CurTask++;                                                  \
            }                                                                       \
            _OS_SET_STATE();/*  16  18   INDF,  30 -  OS_state */ \
            /* ,     */                                \
            if (!_OS_bTaskEnable) goto SCHED_CONTINUE;                              \
            IF_NOT_OS_CHECK_DELAYS_GOTO_SCHED_CONTINUE;                             \
            _OS_JumpToTask();                                                       \
            _OS_SaveRetAddr();                                                      \
        SCHED_CONTINUE:;                                                            \
        }                                                                           

#endif    
/*************************************************************************************************************************/


#endif  // ifndef OS_Shed






#define OS_Run()    for(;;) OS_Sched()


/************************************************************************************
 *
 *   :
 *
 ************************************************************************************/


/************************************************************************/
/* 1.                           */
/************************************************************************/
#define OS_Yield()          _OS_GetReturnPoint()

/************************************************************************/
/* 2.                                */
/*    (  )                                       */
/************************************************************************/
#define _OS_Return()        _OS_ReturnNoSave()

/************************************************************************/
/* 3.   ,          */
/************************************************************************/
#define _OS_ClearReady()                    _OS_CLEAR_READY()

/************************************************************************/
/* 4.   ,    ,          */
/*      ,                                        */
/*                           */
/*                          */
/************************************************************************/
#define _OS_ClearReadySetCanContinue()      _OS_CLEAR_READY_SET_CANCONTINUE()






















//_______________________________________________________________________________________________________________
/****************************************************************************************************************/
/*                                                                                                              */
/*                     T A S K S                                                                                */
/*                                                                                                              */
/****************************************************************************************************************/

/************************************************************************/
/*                                                                      */
/*     (     ) */
/*                                                                      */
/************************************************************************/

#define OS_StopTask()       { _OS_StopTask(); _OS_Return(); }


/************************************************************************/
/*                                                                      */
/*    .          */
/*                                                                      */
/************************************************************************/

#ifdef __OSA18__
    #if OS_CODE_POINTER_SIZE <= 2
        
        #define _OS_SetIndfWord(value)                              \
            _postinc = (((OST_CODE_POINTER)value)) & 0xFF;          \
            _postinc = (((OST_CODE_POINTER)value) >> 8) & 0xFF;

    #else
        #define _OS_SetIndfWord(value)                              \
            _postinc = (((OST_CODE_POINTER)value)) & 0xFF;          \
            _postinc = (((OST_CODE_POINTER)value) >> 8) & 0xFF;     \
            _postinc = (((OST_CODE_POINTER)value) >> 16) & 0xFF;    \
            _postinc = 0;
    #endif
#endif


//------------------------------------------------------------------------------
#if     defined(__OSAMCC30__)

    #define OS_ReplaceTask(priority, TaskName)                                      \
        {                                                                           \
            _OS_StopTask();                                                         \
            *((_OST_INT_TYPE*)&OS_state) = priority | 0x48; /* bEnable  bReady */  \
            _OS_CurTask->pTaskPointer = (OST_CODE_POINTER)TaskName;                 \
            _OS_CurTask->State = OS_state;                                          \
            OS_Flags.bTaskReplaced = 1;                                             \
            _OS_Return();                                                           \
        }


//------------------------------------------------------------------------------
#elif   defined(__OSAMCC18__)

    #define OS_ReplaceTask(priority, TaskName)                                      \
        {                                                                           \
            _OS_StopTask();                                                         \
            _OS_SET_FSR_CUR_TASK();                                                 \
            _postinc = priority | 0x48; /* bEnable  bReady */                      \
            _OS_SetIndfWord(TaskName);                                              \
            _indf = 0;  /* temp FSR1 */                                             \
            _OS_Return();                                                           \
        }

//------------------------------------------------------------------------------
#elif   defined(__OSAPICC12__)
    #define OS_ReplaceTask(priority, TaskName)                                      \
        {                                                                           \
            _OS_StopTask();                                                         \
            *((_OST_INT_TYPE*)&OS_state) = priority | 0x48; /* bEnable  bReady */  \
            _fsr = (_FSR_TYPE*)_OS_CurTask;                                         \
            _indf = priority | 0x48; /* bEnable  bReady */                         \
            _fsr++;                                                                 \
            _indf = ((OST_CODE_POINTER)TaskName) & 0xFF;                            \
            _fsr = 0;                                                               \
            _OS_Return();                                                           \
        }

//------------------------------------------------------------------------------
#elif   defined(__OSAPICC16__)

    #define OS_ReplaceTask(priority, TaskName)                                      \
        {                                                                           \
            _OS_StopTask();                                                         \
            _OS_SET_FSR_CUR_TASK();                                                 \
            _indf = priority | 0x48; /* bEnable  bReady */                         \
            _fsr++;                                                                 \
            _indf = ((OST_CODE_POINTER)TaskName) & 0xFF;                            \
            _fsr++;                                                                 \
            _indf = (((OST_CODE_POINTER)TaskName) >> 8) & 0xFF;                     \
            _OS_Return();                                                           \
        }

//------------------------------------------------------------------------------
#elif   defined(__OSAPICC18__)

    #define OS_ReplaceTask(priority, TaskName)                                      \
        {                                                                           \
            _OS_StopTask();                                                         \
            *((NEAR _OST_INT_TYPE*)&OS_state) = priority | 0x48; /* bEnable  bReady */\
            _OS_SET_FSR_CUR_TASK();                                                 \
            _postinc = priority | 0x48; /* bEnable  bReady */                      \
            _OS_SetIndfWord(TaskName);                                              \
            _OS_Return();                                                           \
        }
//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------




//#define OS_ReplaceTask(priority, TaskAddr)                      \
//    {                                                           \
//        _OS_StopTask();                                         \
//        OS_CreateTask(priority, TaskAddr);                      \
//        _OS_Return();                                           \
//    }        


/****************************************************************************/
/*                                                                          */
/*                                            */
/*      OS_bIsError() == 1,       */
/*    .                                              */
/*                                                                          */
/****************************************************************************/


#if defined(__OSAMCC30__)
    #define OS_CreateTask(priority, TaskName) _OS_CreateTask(priority, (OST_CODE_POINTER)TaskName)
#elif defined (__OSAMCC18__)
    #define OS_CreateTask(priority, TaskName) _OS_CreateTask(priority, (OST_CODE_POINTER)TaskName)
#elif defined(__OSACCS18__)
    #define OS_CreateTask(priority, TaskName)                           \
        {                                                               \
            _OS_CreateTask(priority, (OST_CODE_POINTER)TaskName);       \
            if (_pcl & 1) TaskName();                                   \
        }
#elif defined(__OSAPICC12__)
    #define OS_CreateTask(priority, TaskName)                           \
        {                                                               \
            asm("\t global _main       ");                              \
            asm("\t global _"#TaskName);                                \
            asm("\t fncall _main, _"#TaskName);                         \
            asm("_OS_Add_TaskAddr   _"#TaskName);                       \
            _OS_CreateTask(priority, _OS_temp);                         \
        }

#else
    #define OS_CreateTask(priority, TaskName)                           \
        {                                                               \
            asm("\t global _main       ");                              \
            asm("\t global _" #TaskName);                               \
            asm("\t fncall _main,_" #TaskName);                         \
            _OS_CreateTask(priority, (OST_CODE_POINTER)TaskName);       \
        }
#endif






/************************************************************************/
/*                                                                      */
/*                */
/*                                                                      */
/************************************************************************/

#define OS_CreateTaskP(priority, TaskAddr)      _OS_CreateTask(priority, TaskAddr)


//   ,   OS_CreateTaskP,    
//   main,      
#define OS_ReserveTask(TaskName)                                        \
        {                                                               \
            asm("\t global _main       ");                              \
            asm("\t global _" #TaskName);                               \
            asm("\t fncall _main,_" #TaskName);                         \
        }




/************************************************************************/
/*                                                                      */
/* /                            */
/*                                                                      */
/************************************************************************/

#define OS_GetTaskPriority()                    OS_CurTask->State.Priority
#define OS_SetTaskPriority(new_priority)        {OS_CurTask->State.Priority = new_priority;}

#define OS_GetPriority()                        OS_CurTask->State.Priority
#define OS_SetPriority(new_priority)            {OS_CurTask->State.Priority = new_priority;}

















/************************************************************************/
/*                                                                      */
/*                                       */
/*                                                                      */
/************************************************************************/
#define OS_Delay(delaytime)                            \
        {                                              \
            _OS_InitDelay(delaytime);                  \
            OS_Yield();                                \
        }                                              \
    




















/************************************************************************/
/* ,                                 */
/************************************************************************/
#define OS_IsTimeout()    (OS_Flags.bTimeout)     //    == 0,      

/************************************************************************/
/* ,                                              */
/************************************************************************/
#define OS_IsError()                OS_Flags.bError
#define OS_IsEventError()           OS_Flags.bEventError

/************************************************************************/
/* ,                             */
/************************************************************************/
#define OS_IsInCriticalSection()    OS_Flags.bInCriticalSection











//_______________________________________________________________________________________________________________
/****************************************************************************************************************/
/*                                                                                                              */
/*                     I N T E R R U P T S                                                                      */
/*                                                                                                              */
/****************************************************************************************************************/


//------------------------------------------------------------------------------
//   


#ifdef __OSAMCC30__
    extern _OST_INT_TYPE OS_DI();
    extern void OS_RI(_OST_INT_TYPE);
    #define OS_EI()     SR &= 0xFF1F;
#endif

#ifdef __OSAPICC__
    extern  char OS_DI (void);
    #define OS_RI(cSavedInt)    {if (cSavedInt & 0x80) GIE = 1;}
    #define OS_EI()             { GIE = 1;}
#endif


#ifdef __OSAPICC18__
    extern char OS_DI (void);
    extern void OS_RI (char);
    #define OS_EI()         { GIE = 1; if (IPEN) GIEL = 1;}
    #define OS_EIH()        { GIE = 1; }
    #define OS_EIL()        { GIEL = 1; }
#endif

#ifdef __OSAMCC18__
    extern void _OS_RI (void);
    extern char OS_DI (void);
    #define OS_RI(cSavedInt) {_wreg = cSavedInt; _OS_RI();}
    #define OS_EI()         { GIE = 1; if (IPEN) GIEL = 1;}
    #define OS_EIH()        { GIE = 1; }
    #define OS_EIL()        { GIEL = 1; }
#endif

#ifdef __OSAPICC12__
    #define OS_DI()     0
    #define OS_RI(temp)
#endif


//------------------------------------------------------------------------------
// /  

#ifdef __OSAMCC30__
    #define OS_EnterInt()
    #define OS_LeaveInt()
#else

    /************************************************************************/
    /*                                                                      */
    /*              */
    /*                                                                      */
    /************************************************************************/

    #define OS_EnterInt()                                   \
        static _FSR_TYPE   _OS_FSR_Temp;                    \
        _OS_FSR_Temp = _fsr;                                \

    /************************************************************************/
    /*                                                                      */
    /*                  */
    /*                                                                      */
    /************************************************************************/

    #define OS_LeaveInt()   _fsr = _OS_FSR_Temp

#endif

























//_______________________________________________________________________________________________________________
/****************************************************************************************************************/
/*                                                                                                              */
/*                     C O N D I T I O N S                                                                      */
/*                                                                                                              */
/****************************************************************************************************************/


/***********************************************************************************************
 *
 *       .   
 *      (,   .)   
 *     .
 *
 ***********************************************************************************************/

#if defined(__OSA16__) || defined(__OSA12__)
    extern bit _OS_CheckEvent (char);
    #define _OS_CHECK_EVENT(event) if (!_OS_CheckEvent(event)) _OS_Return();

#elif defined(__OSAMCC18__) 

    extern void _OS_CheckEvent (void);
    #define _OS_CHECK_EVENT(event) {_OS_temp = event; _OS_CheckEvent();}

#else //   C18   C30
    extern void _OS_CheckEvent (_OST_INT_TYPE);
    #define _OS_CHECK_EVENT(event) _OS_CheckEvent(event)

#endif
    
// !!!   :       , .. 
// event -   ,   ,    FSR. 
//       _OS_xxxIndfxxx()
// ..   (   )  :
//  _fsr = (_FSR_TYPE)&_OS_CurTask->State;


/***********************************************************************************************/
#define _OS_WAIT_EVENT(event)                       \
{                                                   \
    _OS_ClearReady();                               \
    _OS_CHECK_EVENT(event);                         \
}

//------------------------------------------------------------------------------

#define _OS_WAIT_EVENT_TIMEOUT(event, timeout)      \
{                                                   \
    _OS_InitDelay(timeout);                         \
    _OS_ClearReadySetCanContinue();                 \
    _OS_CHECK_EVENT(event);                         \
}

/***********************************************************************************************/



/************************************************************************/
/*                                              */
/************************************************************************/
#define OS_WaitCondition(condition)     _OS_WAIT_EVENT(condition)

/************************************************************************/
/*                            */
/************************************************************************/
#define OS_WaitConditionTimeout(condition, timeout)    _OS_WAIT_EVENT_TIMEOUT(condition, timeout)



